﻿#include <QDebug>
#include "bridge.h"
#include "editor.h"
#include "tools.h"
#include "../common/sources/Map.h"

CBridge     bridge;
extern QQuickItem*  tileSelGrid;
extern QQuickItem*  wallSelGrid;
extern QQuickItem*  objSelGrid;

//---------------------------------------------------------------------------------------------
CBridge::CBridge(QObject *parent) : QObject(parent)
{
    // valeurs par défaut
    mMapSize = CVec2(15, 10);
    mMousePos = CVec2(-1, -1);
    leftPressing = false;
    leftPressed = false;
    rightPressed = false;
    mTabIndex = eTabMap;
    mTool = eQMLToolDraw;
    set_tileSelScrollVis(false);
    set_wallSelScrollVis(false);
    set_objectSelScrollVis(false);
    set_monsterSelScrollVis(false);
    set_rectSelScrollVis(false);
    set_tileType(1);
    set_wallType(1);
    set_objectType(1);
    set_monsterType(1);
}

void    CBridge::setMapWidth(qint32 valeur)
{
    mMapSize.x = valeur;
}

void    CBridge::setMapHeight(qint32 valeur)
{
    mMapSize.y = valeur;
}

void    CBridge::setMouse(qint32 x, qint32 y)
{
    leftPressing = false;
    mMousePos = CVec2(x, y);

    editor.useTool();
    updateQMLImage();
}

void    CBridge::setMousePress(qint32 button)
{
    leftPressing = false;

    if (button == Qt::LeftButton)
    {
        leftPressing = true;
        leftPressed = true;
    }
    else if (button == Qt::RightButton)
        rightPressed = true;

    editor.useTool();
    updateQMLImage();
}

void    CBridge::setMouseRelease(qint32 button)
{
    leftPressing = false;

    if (button == Qt::LeftButton)
        leftPressed = false;
    else if (button == Qt::RightButton)
        rightPressed = false;

    editor.useTool();
    updateQMLImage();
}

void CBridge::setTabIndex(qint32 index)
{
    editor.unselect();
    mTabIndex = (ETabIndex)index;
}

void    CBridge::resizeMap()
{
    editor.unselect();
    map.mSize = mMapSize;
    map.resize();
    updateQMLImage();
}

void    CBridge::save(QString fileName)
{
    // astuce pour convertir les anciens formats
#ifdef CONVERT_MAP
    CMap::mTilesParams.clear();
    editor.readTilesDB("databases/tiles.xml");
//    CMap::mWallsParams.clear();
//    editor.readWallsDB("databases/walls.xml");
//    CMap::mObjectsParams.clear();
//    editor.readObjectsDB("databases/items.xml");
#endif

    // [8] pour retirer "file:///"
    map.save(&fileName.toLocal8Bit().data()[8]);
}

void    CBridge::load(QString fileName)
{
    // astuce pour convertir les anciens formats
#ifdef CONVERT_MAP
    CMap::mTilesParams.clear();
    editor.readTilesDB("databases/tiles-old.xml");
//    CMap::mWallsParams.clear();
//    editor.readWallsDB("databases/walls-old.xml");
//    CMap::mObjectsParams.clear();
//    editor.readObjectsDB("databases/items-old.xml");
#endif

    // [8] pour retirer "file:///"
    map.load(&fileName.toLocal8Bit().data()[8]);
    updateQMLImage();
}

void    CBridge::undo()
{
    editor.mUndoStack->undo();
}

void    CBridge::redo()
{
    editor.mUndoStack->redo();
}

void    CBridge::setTool(qint32 tool)
{
    mTool = (EQMLTools)tool;
    updateQMLImage();
}

void    CBridge::copy()
{
    editor.toolCopy();
}

void    CBridge::cut()
{
    editor.toolCut();
}

void    CBridge::setSelTileType(qint32 type)
{
    drawTileTool*   tool = new drawTileTool();

    if (tool->init(editor.mSelectStart, type) == true)
    {
        editor.mUndoStack->push(tool);
        editor.deleteParamItems(&editor.mTileParamItems);
        editor.addTileParamsItems(type);
    }
    else
    {
        delete tool;
    }
}

void    CBridge::setSelWallType(qint32 type)
{
    drawWallTool*   tool = new drawWallTool();

    if (tool->init(editor.mSelectStart, type) == true)
    {
        editor.mUndoStack->push(tool);
        editor.deleteParamItems(&editor.mWallParamItems);
        editor.addWallParamsItems(type);
    }
    else
    {
        delete tool;
    }
}

void    CBridge::setSelMonsterType(qint32 type)
{
    if (type > 0)
    {
        drawMonsterTool*   tool = new drawMonsterTool();

        if (tool->init(editor.mSelectStart, type) == true)
        {
            editor.mUndoStack->push(tool);
            editor.deleteParamItems(&editor.mMonsterParamItems);
            editor.addMonsterParamsItems(type);
        }
        else
        {
            delete tool;
        }
    }
    else if (type == 0)
    {
        map.removeMonsterGroup(editor.mSelectStart / TILE_SIZE);
    }
}


#define SET_COMBO(_enum,_struct) \
    case _enum: \
        ((_struct*)param)->mValue = value; \
        break;

void    CBridge::setSelParamCombo(qint32 id, qint32 value)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabTiles)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        CParam*     param = map.getTile(pos)->mParams[id];

        switch (param->mType)
        {
            SET_COMBO(eParamOrnate, CParamOrnate)
            SET_COMBO(eParamDoor, CParamDoor)
            SET_COMBO(eParamDoorOrnate, CParamDoorOrnate)
            SET_COMBO(eParamEnum, CParamEnum)

            default:
                break;
        }
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CWall*      wall = &map.getTile(pos)->mWalls[side];
        bool        isStack = false;

        if ((size_t)id >= wall->mParams.size())
            isStack = true;
        else if (wall->mParams[id]->mType == eParamStack)
            isStack = true;

        if (isStack == true)
        {
            CObjectStack*   stack = map.findObjectsStack(pos, side);

            if (stack != NULL)
            {
                CObject&    object = stack->getObject(id - (wall->mParams.size() - 1));
                int oldType = object.getType();
                object.setType(value);

                // met à jour les boutons
                if (oldType != value)
                    QMetaObject::invokeMethod(this, "updateSelStack", Qt::QueuedConnection);
            }
        }
        else
        {
            CParam*     param = wall->mParams[id];

            switch (param->mType)
            {
                SET_COMBO(eParamOrnate, CParamOrnate)
                SET_COMBO(eParamEnum, CParamEnum)

                default:
                    break;
            }
        }
    }
    else if (mTabIndex == eTabObjects)
    {
        CVec2       pos = editor.mSelectStart / (TILE_SIZE / 2);
        CObjectStack*   stack = map.findObjectsStack(pos, eWallSideMax);

        if (stack != NULL)
        {
            CObject&    object = stack->getObject(id);
			int oldType = object.getType();
            object.setType(value);

			// met à jour les boutons
            if (oldType != value)
                QMetaObject::invokeMethod(this, "updateSelStack", Qt::QueuedConnection);
        }
    }
}

void    CBridge::setSelParamText(qint32 id, QString value)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabTiles)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        CTile*      tile = map.getTile(pos);
        CParam*     param = tile->mParams[id];

        switch (param->mType)
        {
            case eParamInt:
                ((CParamInt*)param)->mValue = value.toInt();
                break;

            case eParamScript:
                ((CParamScript*)param)->mValue = value.toUtf8().constData();
                break;

            default:
                break;
        }
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);

        if (id >= 0x10000)
        {
            CObjectStack*   stack = map.findObjectsStack(pos, side);
            int objNum = id & 0xFFFF;
            int paramNum = (id / 0x10000) - 1;
            CObject&    obj = stack->getObject(objNum);
            CParamType  paramInfos = CMap::mObjectsParams[obj.getType()][paramNum];

            if (paramInfos.mType == eParamString)
            {
                obj.setStringParam(paramInfos.mName, value.toLocal8Bit().constData());
            }
            else if (paramInfos.mType == eParamInt)
            {
                obj.setIntParam(paramInfos.mName, value.toInt());
            }
        }
        else
        {
            CWall*      wall = &map.getTile(pos)->mWalls[side];
            CParam*     param = wall->mParams[id];

            switch (param->mType)
            {
                case eParamInt:
                    ((CParamInt*)param)->mValue = value.toInt();
                    break;

                case eParamScript:
                    ((CParamScript*)param)->mValue = value.toUtf8().constData();
                    break;

                case eParamString:
                    ((CParamString*)param)->mValue = value.toUtf8().constData();
                    break;

                default:
                    break;
            }
        }
    }
    else if (mTabIndex == eTabObjects)
    {
        CVec2       pos = editor.mSelectStart / (TILE_SIZE / 2);

        if (id >= 0x10000)
        {
            CObjectStack*   stack = map.findObjectsStack(pos, eWallSideMax);
            int objNum = id & 0xFFFF;
            int paramNum = (id / 0x10000) - 1;
            CObject&    obj = stack->getObject(objNum);
            CParamType  paramInfos = CMap::mObjectsParams[obj.getType()][paramNum];

            if (paramInfos.mType == eParamString)
            {
                obj.setStringParam(paramInfos.mName, value.toLocal8Bit().constData());
            }
            else if (paramInfos.mType == eParamInt)
            {
                obj.setIntParam(paramInfos.mName, value.toInt());
            }
        }
    }
}

void    CBridge::setSelParamBool(qint32 id, bool value)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabTiles)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        CTile*      tile = map.getTile(pos);
        CParam*     param = tile->mParams[id];

        switch (param->mType)
        {
            case eParamBool:
                ((CParamBool*)param)->mValue = value;
                break;

            default:
                break;
        }
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CWall*      wall = &map.getTile(pos)->mWalls[side];
        CParam*     param = wall->mParams[id];

        switch (param->mType)
        {
            case eParamBool:
                ((CParamBool*)param)->mValue = value;
                break;

            default:
                break;
        }
    }
}

void    CBridge::selButtonClicked(qint32 id)
{
    if (id == -1)
        return;

    if (mTabIndex == eTabObjects)
    {
        CVec2       pos = editor.mSelectStart / (TILE_SIZE / 2);
        CObjectStack*   stack = map.findObjectsStack(pos, eWallSideMax);

        if (stack == NULL || (size_t)id == stack->getSize())
        {
            // ajoute un nouvel objet
            if (stack == NULL)
            {
                stack = map.addObjectsStack(pos, eWallSideMax);
                updateQMLImage();
            }

            CObject object;
            stack->addObject(object);
        }
        else
        {
            // retire un objet
            stack->removeObject(id);

            if (stack->getSize() == 0)
            {
                map.removeObjectsStack(pos, eWallSideMax);
                updateQMLImage();
            }
        }

        // met à jour les boutons
        QMetaObject::invokeMethod(this, "updateSelStack", Qt::QueuedConnection);
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CObjectStack*   stack = map.findObjectsStack(pos, side);

        if (stack == NULL || (size_t)id == stack->getSize())
        {
            // add new object
            if (stack == NULL)
                stack = map.addObjectsStack(pos, side);

            CObject object;
            stack->addObject(object);
        }
        else
        {
            // remove object
            stack->removeObject(id);

            if (stack->getSize() == 0)
                map.removeObjectsStack(pos, side);
        }

		// met à jour les boutons
        QMetaObject::invokeMethod(this, "updateSelStack", Qt::QueuedConnection);
    }
}

void    CBridge::updateSelStack()
{
    if (mTabIndex == eTabObjects)
    {
        editor.deleteParamItems(&editor.mObjParamItems);
        editor.addObjParamsItems();
    }
    else if (mTabIndex == eTabWalls)
    {
        CVec2       pos = editor.mSelectStart / TILE_SIZE;
        EWallSide   side = editor.getWallSideAbs(editor.mSelectStart);
        CWall*      wall = &map.getTile(pos)->mWalls[side];
        editor.deleteParamItems(&editor.mWallParamItems);
        editor.addWallParamsItems(wall->getType());
    }
}

//---------------------------------------------------------------------------------------------
MyImageProvider::MyImageProvider() : QQuickImageProvider(QQuickImageProvider::Image)
{
}

QImage MyImageProvider::requestImage(const QString &/*id*/, QSize *size, const QSize &/*requestedSize*/)
{
    QImage image = editor.displayMap(map, bridge.mMousePos, bridge.mTabIndex);

    if (size != NULL)
        *size = QSize(image.width(), image.height());

    return image;
}
