﻿#include "clipboard.h"
#include "editor.h"

CClipboard::CClipboard()
{
    mMem = NULL;
}

CClipboard::~CClipboard()
{
    clear();
}

void CClipboard::clear()
{
    if (mMem != NULL)
        delete[] mMem;

    mObjectsStacks.clear();
    mMonsterGroups.clear();
}

bool CClipboard::isEmpty()
{
    if ((bridge.mTabIndex == eTabTiles || bridge.mTabIndex == eTabWalls) && mMem == NULL)
        return true;
    if (bridge.mTabIndex == eTabObjects && mObjectsStacks.size() == 0)
        return true;
    if (bridge.mTabIndex == eTabMonsters && mMonsterGroups.size() == 0)
        return true;

    return false;
}

void CClipboard::copyFromMap(CVec2 mousePos, CVec2 size, ETabIndex tabIndex)
{
    CVec2 pos = mousePos / TILE_SIZE;

    if (pos.x + size.x > map.mSize.x)
        size.x = map.mSize.x - pos.x;

    if (pos.y + size.y > map.mSize.y)
        size.y = map.mSize.y - pos.y;

    if (size.x == 0 || size.y == 0)
        return;

    mType = eTabTiles;

    switch (tabIndex)
    {
    case eTabWalls:
        if (editor.isSelectSingleWall() == true)
        {
            mType = eTabWalls;
            mSide = editor.getWallSideAbs(mousePos);
        }
        break;

    case eTabObjects:
        if (editor.isSelectSingleObject() == true)
        {
            mType = eTabObjects;
            mObjPos = (mousePos / (TILE_SIZE / 2)) % 2;
        }
        break;

    default:
        break;
    }

    clear();
    mSize = size;

    // copie les cases et les murs
    if (tabIndex == eTabTiles || tabIndex == eTabWalls)
    {
        mMem = new CTile[mSize.x * mSize.y];

        if (mMem != NULL)
            for (int y = 0; y < mSize.y; ++y)
                for (int x = 0; x < mSize.x; ++x)
                    mMem[y * mSize.x + x] = *map.getTile(CVec2(pos.x + x, pos.y + y));
    }

    // copie les objets dans les murs
    if (tabIndex == eTabWalls)
    {
        for (int y = 0; y < mSize.y; ++y)
            for (int x = 0; x < mSize.x; ++x)
                for (int side = 0; side < eWallSideMax; ++side)
                {
                    CVec2   pos3(x, y);
                    CVec2   pos2 = pos + pos3;
                    CObjectStack*   stack = map.findObjectsStack(pos2, (EWallSide)side);

                    if (stack != NULL)
                    {
                        mObjectsStacks.push_back(*stack);
                        mObjectsStacks[mObjectsStacks.size() - 1].mPos = pos3;
                    }
                }
    }

    // copie les objets au sol
    if (tabIndex == eTabObjects)
    {
        for (int y = 0; y < mSize.y * 2; ++y)
            for (int x = 0; x < mSize.x * 2; ++x)
            {
                CVec2   pos3(x, y);
                CVec2   pos2 = pos * 2 + pos3;
                CObjectStack*   stack = map.findObjectsStack(pos2, eWallSideMax);

                if (stack != NULL)
                {
                    mObjectsStacks.push_back(*stack);
                    mObjectsStacks[mObjectsStacks.size() - 1].mPos = pos3;
                }
            }
    }

    // copie les monstres
    if (tabIndex == eTabMonsters)
    {
        for (int y = 0; y < mSize.y; ++y)
            for (int x = 0; x < mSize.x; ++x)
            {
                CVec2   pos3(x, y);
                CVec2   pos2 = pos + pos3;
                CMonsterGroup*  group = map.findMonsterGroup(pos2);

                if (group != NULL)
                {
                    mMonsterGroups.push_back(*group);
                    mMonsterGroups[mMonsterGroups.size() - 1].mPos = pos3;
                }
            }
    }
}

void CClipboard::copyFromClip(CClipboard* src)
{
    clear();

    if (src->isEmpty() == true)
        return;

    mType = src->mType;
    mSide = src->mSide;
    mSize = src->mSize;
    mObjPos = src->mObjPos;
    mMem = new CTile[mSize.x * mSize.y];

    if (mMem != NULL && src->mMem != NULL)
        for (int i = 0; i < mSize.x * mSize.y; ++i)
            mMem[i] = src->mMem[i];

    mObjectsStacks = src->mObjectsStacks;
    mMonsterGroups = src->mMonsterGroups;
}

void CClipboard::copyToMap(CVec2 mousePos, ETabIndex index)
{
    CVec2   pos = mousePos / TILE_SIZE;

    if (mType == eTabTiles || mType == eTabMonsters)
    {
        // une ou plusieurs cases
        if (index == eTabTiles || index == eTabWalls)
        {
            // copie les cases et les murs
            for (int y = 0; y < mSize.y; ++y)
                for (int x = 0; x < mSize.x; ++x)
                {
                    CTile*  tile = map.getTile(CVec2(pos.x + x, pos.y + y));

                    if (tile != NULL)
                    {
                        if (index == eTabTiles)
                        {
                            tile->copyWithoutWalls(mMem[y * mSize.x + x]);
                        }
                        else if (index == eTabWalls)
                        {
                            for (int i = 0; i < eWallSideMax; ++i)
                                tile->mWalls[i] = mMem[y * mSize.x + x].mWalls[i];
                        }
                    }
                }

            // copie les objets dans les murs
            if (index == eTabWalls)
            {
                for (int y = 0; y < mSize.y; ++y)
                    for (int x = 0; x < mSize.x; ++x)
                        for (int side = 0; side < eWallSideMax; ++side)
                        {
                            CVec2   mapPos = pos + CVec2(x, y);
                            map.removeObjectsStack(mapPos, (EWallSide)side);
                        }

                for (size_t i = 0; i < mObjectsStacks.size(); ++i)
                {
                    CVec2   mapPos = pos + mObjectsStacks[i].mPos;
                    map.mObjectsStacks.push_back(mObjectsStacks[i]);
                    map.mObjectsStacks[map.mObjectsStacks.size() - 1].mPos = mapPos;
                }
            }
        }
        else if (index == eTabObjects)
        {
            // copie les objets sur le sol
            for (int y = 0; y < mSize.y * 2; ++y)
                for (int x = 0; x < mSize.x * 2; ++x)
                {
                    CVec2   mapPos = pos * 2 + CVec2(x, y);
                    map.removeObjectsStack(mapPos, eWallSideMax);
                }

            for (size_t i = 0; i < mObjectsStacks.size(); ++i)
            {
                CVec2   mapPos = pos * 2 + mObjectsStacks[i].mPos;
                map.mObjectsStacks.push_back(mObjectsStacks[i]);
                map.mObjectsStacks[map.mObjectsStacks.size() - 1].mPos = mapPos;
            }
        }
        else if (index == eTabMonsters)
        {
            // copie les monstres
            for (int y = 0; y < mSize.y; ++y)
                for (int x = 0; x < mSize.x; ++x)
                {
                    CVec2   mapPos = pos + CVec2(x, y);
                    map.removeMonsterGroup(mapPos);
                }

            for (size_t i = 0; i < mMonsterGroups.size(); ++i)
            {
                CVec2   mapPos = pos + mMonsterGroups[i].mPos;
                map.mMonsterGroups.push_back(mMonsterGroups[i]);
                map.mMonsterGroups[map.mMonsterGroups.size() - 1].mPos = mapPos;
            }
        }
    }
    else if (mType == eTabWalls)
    {
        // un mur
        CTile*      tile = map.getTile(pos);
        EWallSide   side = editor.getWallSideAbs(mousePos);
        tile->mWalls[side] = mMem[0].mWalls[mSide];
        map.removeObjectsStack(pos, side);

        for (size_t i = 0; i < mObjectsStacks.size(); ++i)
        {
            if (mObjectsStacks[i].mSide == mSide)
            {
                CObjectStack*   stack = map.addObjectsStack(pos, side);
                *stack = mObjectsStacks[i];
                stack->mPos = pos;
                stack->mSide = side;
            }
        }
    }
    else if (mType == eTabObjects)
    {
        // une case d'objets
        pos = mousePos / (TILE_SIZE / 2);
        map.removeObjectsStack(pos, eWallSideMax);

        for (size_t i = 0; i < mObjectsStacks.size(); ++i)
        {
            if (mObjectsStacks[i].mPos == mObjPos)
            {
                map.mObjectsStacks.push_back(mObjectsStacks[i]);
                map.mObjectsStacks[map.mObjectsStacks.size() - 1].mPos = pos;
            }
        }
    }
}
